module Benchmark.AMR exposing (main)

import AMR.WithDenseList as WithDenseList
import AMR.WithList as WithList
import Benchmark exposing (Benchmark, benchmark)
import Benchmark.Fixture as Fixture
import Benchmark.Runner exposing (BenchmarkProgram)
import Json.Decode
import Json.Encode


main : BenchmarkProgram
main =
    Benchmark.Runner.program <|
        Benchmark.describe "AMR"
            [ init
            , update
            , encode
            , decoder
            ]


init : Benchmark
init =
    let
        fromCacheWithList =
            Fixture.modelWithList
                |> WithList.encode Fixture.encodeMsg
                |> Json.Encode.encode 0
                |> Just

        fromCacheWithDenseList =
            Fixture.modelWithDenseList
                |> WithDenseList.encode
                |> Json.Encode.encode 0
                |> Just
    in
    Benchmark.compare "init"
        "withDenseList"
        (\_ ->
            WithDenseList.init
                Fixture.update
                Fixture.msgDecoder
                fromCacheWithDenseList
                Fixture.init
        )
        "withList"
        (\_ ->
            WithList.init
                Fixture.update
                Fixture.msgDecoder
                fromCacheWithList
                Fixture.init
        )


update : Benchmark
update =
    Benchmark.compare "update"
        "withDenseList"
        (\_ ->
            WithDenseList.update
                Fixture.update
                (always Cmd.none)
                Fixture.encodeMsg
                Fixture.Increment
                Fixture.modelWithDenseList
        )
        "withList"
        (\_ ->
            WithList.update
                Fixture.update
                (always Cmd.none)
                Fixture.encodeMsg
                Fixture.Increment
                Fixture.modelWithList
        )


encode : Benchmark
encode =
    Benchmark.compare "encode"
        "withDenseList"
        (\_ ->
            WithDenseList.encode Fixture.modelWithDenseList
        )
        "withList"
        (\_ ->
            WithList.encode
                Fixture.encodeMsg
                Fixture.modelWithList
        )


decoder : Benchmark
decoder =
    let
        encodedWithList =
            WithList.encode Fixture.encodeMsg Fixture.modelWithList

        withListDecoder =
            WithList.decoder
                (\msg model_ -> Tuple.first (Fixture.update msg model_))
                Fixture.msgDecoder
                (Tuple.first Fixture.init)

        encodedWithDenseList =
            WithDenseList.encode Fixture.modelWithDenseList

        withDenseListDecoder =
            WithDenseList.decoder
                (\msg model_ -> Tuple.first (Fixture.update msg model_))
                Fixture.msgDecoder
                (Tuple.first Fixture.init)
    in
    Benchmark.compare "decoder"
        "withDenseList"
        (\_ -> Json.Decode.decodeValue withDenseListDecoder encodedWithDenseList)
        "withList"
        (\_ -> Json.Decode.decodeValue withListDecoder encodedWithList)
