module AMR.WithList exposing (Model, decoder, encode, init, update)

import Dict exposing (Dict)
import Json.Decode
import Json.Encode


type alias Model model msg =
    { appModel : model
    , appMsgs : List msg
    }


init :
    (msg -> model -> ( model, Cmd msg ))
    -> Json.Decode.Decoder msg
    -> Maybe String
    -> ( model, Cmd msg )
    -> ( Model model msg, Cmd msg )
init updateApp msgDecoder fromCache ( appModel, appCmd ) =
    case Maybe.map (Json.Decode.decodeString (decoder (withoutCmd updateApp) msgDecoder appModel)) fromCache of
        Just (Ok model) ->
            ( model
            , Cmd.none
            )

        _ ->
            ( { appModel = appModel
              , appMsgs = []
              }
            , appCmd
            )


update :
    (msg -> model -> ( model, Cmd msg ))
    -> (String -> Cmd msg)
    -> (msg -> Json.Encode.Value)
    -> msg
    -> Model model msg
    -> ( Model model msg, Cmd msg )
update updateApp toCache encodeMsg appMsg model =
    let
        ( appModel, appCmd ) =
            updateApp appMsg model.appModel

        updatedModel =
            { model
                | appModel = appModel
                , appMsgs = appMsg :: model.appMsgs
            }
    in
    ( updatedModel
    , Cmd.batch
        [ appCmd
        , toCache (Json.Encode.encode 0 (encode encodeMsg updatedModel))
        ]
    )


encode : (msg -> Json.Encode.Value) -> Model model msg -> Json.Encode.Value
encode encodeMsg model =
    Json.Encode.list encodeMsg model.appMsgs


decoder :
    (msg -> model -> model)
    -> Json.Decode.Decoder msg
    -> model
    -> Json.Decode.Decoder (Model model msg)
decoder replayUpdate msgDecoder appModel =
    Json.Decode.map
        (\appMsgs ->
            { appModel = List.foldr replayUpdate appModel appMsgs
            , appMsgs = appMsgs
            }
        )
        (Json.Decode.list msgDecoder)


withoutCmd : (msg -> model -> ( model, Cmd msg )) -> msg -> model -> model
withoutCmd updateApp msg model =
    Tuple.first (updateApp msg model)
